/**
* Dimensions middleware
*
* EXAMPLE:
* Use it with no parameters to get all the dimensions specified
* app.get('/', dimensions(), function(req, res, next){})
*
* Use it with an array of accepted values
* app.get('/', dimensions(['year', 'location']), function(req, res, next){})
*
* Use it globally
* app.use(dimensions())
*/

/**
 * This function returns the intersection of two arrays
 * @param  {array} a [description]
 * @param  {array} b [description]
 * @return {array}   [description]
 */
function intersect(a, b) {
    let t;
    if (b.length > a.length) {
        t = b; b = a; a = t;
    }
    return a.filter((e) => b.indexOf(e) !== -1);
}

function dimensions(dims) {
    return (req, res, next) => {
        req.dims = {};
        if (req.query.dims) {
            const params = req.query.dims.split(',');
            const dimObj = {};
            for (const param of params) {
                const kv = param.split(':');
                dimObj[kv[0]] = (typeof kv[1] === 'undefined') ? null : kv[1];
            }
            // for(let i=0; i<params.length; ++i) {
            //     let kv = params[i].split(':');
            //     dimObj[kv[0]] = (typeof kv[1] === 'undefined') ? null : kv[1];
            // }

            // If the dims array exists and is not empty
            if (typeof dims !== 'undefined' && dims.length > 0) {
                const intersection = intersect(dims, Object.keys(dimObj));
                for (let i = 0; i < intersection.length; ++i) {
                    req.dims[intersection[i]] = dimObj[intersection[i]];
                }
            } else {
                req.dims = dimObj;
            }
        }
        next();
    };
}

module.exports = dimensions;
